//+------------------------------------------------------------------+
//|                                              ChannelsTrader3.mq5 |
///|                                          Copyright 2016, denkir |
//|                             https://www.mql5.com/ru/users/denkir |
//+------------------------------------------------------------------+
#property copyright "Copyright 2016, denkir"
#property link      "https://www.mql5.com/ru/users/denkir"
#property version   "1.00"
//+------------------------------------------------------------------+
//| Include                                                          |
//+------------------------------------------------------------------+
#include "EquidistantChannelExpert3.mqh"
#include "SignalEquidChannel3.mqh"
#include <Expert\Trailing\TrailingNone.mqh>
#include <Expert\Money\MoneyFixedLot.mqh>
//+------------------------------------------------------------------+
//| Inputs                                                           |
//+------------------------------------------------------------------+
sinput string Info_trade="+===--  --====+"; // +===--  --====+
input double InpStopLevel=50.0;         // -, 
input double InpTakeLevel=50.0;         // -, 
sinput ulong InpMagicNumber=777;        // 
//---
sinput string Info_signals="+===--  --====+"; // +===--  --====+
input bool InpOnStart=false;            //   ?
input int InpThresholdOpen=10;          //    , [0...100]
input double InpSignalWeight=1.0;       // , [0...1.0]
//---
sinput string Info_money="+===--   --====+"; // +===--   --====+
input double InpFixLotPercent=10.0;     // 
input double InpLots=0.1;               //  
//---
sinput string Info_channels="+===--  --====+"; // +===--  --====+
input int InpPipsInside=100;            //  , 
input int InpPipsOutside=150;           //  , 
input int InpNarrowPips=250;            //  , 
input int InpBroadPips=1200;            //  , 
input int InpPrevFracNum=3;             //  
input bool InpToPlotFracs=true;         //  ?
input int InpBarsBeside=10;             //  /  
input int InpBarsBetween=1;             //  
input ENUM_RELEVANT_EXTREMUM InpRelevantPoint=RELEVANT_EXTREMUM_PREV; //  
sinput int InpLineWidth=3;              //  
sinput bool InpToLog=true;              //  ?
//+------------------------------------------------------------------+
//| Global expert object                                             |
//+------------------------------------------------------------------+
CEquidChannelExpert myChannelExpert;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//---  
   if(!myChannelExpert.Init(Symbol(),Period(),true,InpMagicNumber))
     {
      //--- 
      PrintFormat(__FUNCTION__+": error initializing expert");
      return INIT_FAILED;
     }
//---  
   CExpertSignal *signal=new CExpertSignal;
   if(signal==NULL)
     {
      //--- 
      PrintFormat(__FUNCTION__+": error creating signal");
      return INIT_FAILED;
     }
//---  
   myChannelExpert.InitSignal(signal);
   signal.ThresholdOpen(InpThresholdOpen);
   signal.StopLevel(InpStopLevel);
   signal.TakeLevel(InpTakeLevel);
   signal.Expiration(0);
//---  CSignalEquidChannel
   CSignalEquidChannel *filter0=new CSignalEquidChannel;
   if(filter0==NULL)
     {
      //--- 
      PrintFormat(__FUNCTION__+": error creating filter0");
      return INIT_FAILED;
     }
   signal.AddFilter(filter0);
//---  
   filter0.PointsInside(_Point*InpPipsInside);
   filter0.PointsOutside(_Point*InpPipsOutside);
   if(InpNarrowPips>=InpBroadPips)
     {
      PrintFormat(__FUNCTION__+": error specifying narrow and broad values");
      return INIT_FAILED;
     }
   filter0.NarrowTolerance(_Point*InpNarrowPips);
   filter0.BroadTolerance(_Point*InpBroadPips);      
   filter0.PrevFracNum(InpPrevFracNum);
   filter0.ToPlotFracs(InpToPlotFracs);
   filter0.BarsBeside(InpBarsBeside);
   filter0.BarsBetween(InpBarsBetween);
   filter0.RelevantPoint(InpRelevantPoint);
   filter0.LineWidth(InpLineWidth);
   filter0.ToLog(InpToLog);
   filter0.SignalOnStart(InpOnStart);
   filter0.Weight(InpSignalWeight);
   filter0.Pattern_0(90);
//---   
   CTrailingNone *trailing=new CTrailingNone;
   if(trailing==NULL)
     {
      //--- 
      PrintFormat(__FUNCTION__+": error creating trailing");
      return INIT_FAILED;
     }
//---    (will be deleted automatically))
   if(!myChannelExpert.InitTrailing(trailing))
     {
      //--- 
      PrintFormat(__FUNCTION__+": error initializing trailing");
      return INIT_FAILED;
     }
//---   
   CMoneyFixedLot *money=new CMoneyFixedLot;
   if(money==NULL)
     {
      //--- 
      PrintFormat(__FUNCTION__+": error creating money");
      return INIT_FAILED;
     }
//---     (will be deleted automatically))
   if(!myChannelExpert.InitMoney(money))
     {
      //--- 
      PrintFormat(__FUNCTION__+": error initializing money");
      return INIT_FAILED;
     }
//---    
   money.Percent(InpFixLotPercent);
   money.Lots(InpLots);
//---   
   if(!myChannelExpert.ValidationSettings())
     {
      //--- 
      return INIT_FAILED;
     }
//---   
   if(!myChannelExpert.InitIndicators())
     {
      //--- 
      PrintFormat(__FUNCTION__+": error initializing indicators");
      return INIT_FAILED;
     }
//---
   return INIT_SUCCEEDED;
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---  
   myChannelExpert.Deinit();
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---   
   myChannelExpert.OnTick();
  }
//+------------------------------------------------------------------+
