//+------------------------------------------------------------------+
//|                                         TrailingEquidChannel.mqh |
//|                                           Copyright 2016, denkir |
//|                             https://www.mql5.com/ru/users/denkir |
//+------------------------------------------------------------------+
#property copyright "Copyright 2016, denkir"
#property link      "https://www.mql5.com/ru/users/denkir"
//--- include
#include <Expert\ExpertTrailing.mqh>
#include "SignalEquidChannel4.mqh"
//+------------------------------------------------------------------+
//| Class CTrailingEquidChannel.                                     |
//| Purpose: Class of trailing stops based on Equidistant Channel.   |
//|              Derives from class CExpertTrailing.                 |
//+------------------------------------------------------------------+
class CTrailingEquidChannel : public CExpertTrailing
  {
protected:
   double            m_sl_distance;       //   
   double            m_tp_distance;       //   
   double            m_upper_val;         //  
   double            m_lower_val;         //  
   ENUM_CHANNEL_WIDTH_TYPE m_ch_wid_type; //    
   //---
public:
   void              CTrailingEquidChannel(void);
   void             ~CTrailingEquidChannel(void){};
   //--- methods of initialization of protected data
   void              SetTradeLevels(double _sl_distance,double _tp_distance);
   //---
   virtual bool      CheckTrailingStopLong(CPositionInfo *position,double &sl,double &tp);
   virtual bool      CheckTrailingStopShort(CPositionInfo *position,double &sl,double &tp);
   //---
   bool              RefreshData(const CSignalEquidChannel *_ptr_ch_signal);
  };
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
void CTrailingEquidChannel::CTrailingEquidChannel(void)
  {
   m_sl_distance=m_tp_distance=m_upper_val=m_lower_val=0.;
  }
//+------------------------------------------------------------------+
//|                                                    |
//+------------------------------------------------------------------+
void CTrailingEquidChannel::SetTradeLevels(double _sl_distance,double _tp_distance)
  {
   m_sl_distance=_sl_distance;       //   
   m_tp_distance=_tp_distance;       //   
  }
//+------------------------------------------------------------------+
//| Checking trailing stop and/or profit for long position.          |
//+------------------------------------------------------------------+
bool CTrailingEquidChannel::CheckTrailingStopLong(CPositionInfo *position,double &sl,double &tp)
  {
//--- check
   if(position==NULL)
      return false;
//--- -       
   double level=m_symbol.NormalizePrice(m_symbol.Bid()-m_symbol.StopsLevel()*m_symbol.Point());
   if(m_lower_val>0.)
     {
      double new_sl=m_symbol.NormalizePrice(m_lower_val-m_sl_distance*PriceLevelUnit());
      double pos_sl=position.StopLoss();
      double pos_tp=position.TakeProfit();
      sl=tp=EMPTY_VALUE;
      //--- SL
      if(new_sl<level)
         if(new_sl!=pos_sl)
           {
            //---  sl-
            sl=new_sl;
            //--- TP
            double new_tp;
            //---   
            if(m_ch_wid_type==CHANNEL_WIDTH_NARROW)
              {
               double pos_opr=position.PriceOpen();
               new_tp=m_symbol.NormalizePrice(pos_opr+2.*(m_upper_val-m_lower_val));
              }
            else
              {
               new_tp=m_symbol.NormalizePrice(m_upper_val);
              }
            //--- 
            level=m_symbol.NormalizePrice(m_symbol.Bid()+m_symbol.StopsLevel()*m_symbol.Point());
            if(new_tp>level)
              {
               if(new_tp!=pos_tp)
                  tp=new_tp;
               else
                  tp=pos_tp;
              }
           }
     }
//---
   return(sl!=EMPTY_VALUE);
  }
//+------------------------------------------------------------------+
//| Checking trailing stop and/or profit for short position.         |
//+------------------------------------------------------------------+
bool CTrailingEquidChannel::CheckTrailingStopShort(CPositionInfo *position,double &sl,double &tp)
  {
//--- check
   if(position==NULL)
      return false;
//--- -       
   double level=m_symbol.NormalizePrice(m_symbol.Ask()+m_symbol.StopsLevel()*m_symbol.Point());
   if(m_upper_val>0.)
     {
      double new_sl=m_symbol.NormalizePrice(m_upper_val+m_sl_distance*PriceLevelUnit()+
                                    m_symbol.Spread()*m_symbol.Point());
      double pos_sl=position.StopLoss();
      double pos_tp=position.TakeProfit();
      sl=tp=EMPTY_VALUE;
      //--- SL
      if(new_sl>level)
         if(new_sl!=pos_sl)
           {
            //---  sl-
            sl=new_sl;
            //--- TP
            double new_tp;
            //---   
            if(m_ch_wid_type==CHANNEL_WIDTH_NARROW)
              {
               double pos_opr=position.PriceOpen();
               new_tp=m_symbol.NormalizePrice(pos_opr-2.*(m_upper_val-m_lower_val)+
                                      m_symbol.Spread()*m_symbol.Point());
              }
            else
              {
               new_tp=m_symbol.NormalizePrice(m_lower_val);
              }
            //--- 
            level=m_symbol.NormalizePrice(m_symbol.Ask()-m_symbol.StopsLevel()*m_symbol.Point());
            if(new_tp<level)
              {
               if(new_tp!=pos_tp)
                  tp=new_tp;
               else
                  tp=pos_tp;
              }
           }
     }
//---
   return(sl!=EMPTY_VALUE);
  }
//+------------------------------------------------------------------+
//|                                            |
//+------------------------------------------------------------------+
bool CTrailingEquidChannel::RefreshData(const CSignalEquidChannel *_ptr_ch_signal)
  {
//---     
   if(_ptr_ch_signal==NULL)
      return false;
//--- 1)  
   if(!_ptr_ch_signal.Borders(m_upper_val,m_lower_val))
      return false;
//--- 2)    
   m_ch_wid_type=_ptr_ch_signal.WidthType();
//---
   return true;
  }
//+------------------------------------------------------------------+
